<?php
namespace CADSUP\Controllers;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Geral\Logic\ParceiroNegocio;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use CADSUP\Services\Sup00800FornecedorService;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Sup00800FornecedorController extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $dataSourceName;
    protected $environment;

    public function __construct($crudService, $parameterBag, $connection, Sup00800FornecedorService $service, $environment) {
        $this->service = $service;
        $this->environment = $environment;
        parent::__construct($crudService, $parameterBag);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) 
    {
        $dataSource = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()));
     
        if($dataSource === 'fornecedor_visualizacao') {
            if( $this->service->getFranqueadora() ) {
                $this->dataSourceName = 'fornecedor_visualizacao_filial';

                $conditions = $request->getFilterCriteria()->getConditions();
                $request->getFilterCriteria()->addCondition("P_CDFILIAL", "=", $this->environment->getCdfilial());
    
            }
            else {
                $this->dataSourceName = $dataSource;
            }
        }
        else {
            $this->dataSourceName = $dataSource;
        }
 
        parent::find($request, $response);
    }

    public function save(DTO\Request $request, DTO\Response $response) 
    {
        $dataSource = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()) - 6);
        
        if($dataSource === 'fornecedor_visualizacao') {
            $this->saveFornecedor($request, $response);
        }
        else {
            $this->dataSourceName = $dataSource;
            parent::save($request, $response);
        } 
        

    }
    
    public function saveFornecedor(DTO\Request $request, DTO\Response $response) 
    {
        $dataSource = 'fornecedor_visualizacao';
        
        $filial = $this->environment->getCdfilial();
        
        $franqueadora = $this->service->getFranqueadora();

        if( $franqueadora ) {
            $dataSource = 'fornecedor_visualizacao_filial';
            $dataSet = $request->getDataSet();
            $rows = $dataSet->getRows();

            if( $franqueadora != $this->environment->getCdfilial()) {
                
                foreach($rows as &$row) {
                    
                    $fornecedor         = $row["CDFORNECED"];
                    $row["IDTPFORN"]    = 'L';
                    
                    $existeFornecedor   = $this->service->existeFornecedor($fornecedor);
                
                    if(!$existeFornecedor) {
                        $this->dataSourceName = $dataSource;
                        parent::save($request, $response);
                    }

                    if(!$this->service->getFiliForn($filial, $fornecedor)) {
                        $this->service->insertFiliForn($filial, $fornecedor);
                    }
                    else {
                        throw new \Exception("Fornecedor já cadastrado.");
                    }
                }
            }
            else {
                $dataSource = 'fornecedor_visualizacao';
                $this->dataSourceName = $dataSource;
                parent::save($request, $response);

            }
        }
        else {
            $this->dataSourceName = $dataSource;
            parent::save($request, $response);
        }
    }

    public function delete(DTO\Request $request, DTO\Response $response) 
    {
        $dataSource = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()) - 8);
        $this->dataSourceName = $dataSource;
        parent::delete($request, $response);
    }
    
    public function deleteFormaPagamento(DTO\Request\Row $request, DTO\Response $response) 
    {
        try {
            $this->service->deleteFormaPagamento($request->getRow());
        }
        catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function saveContaCorrente(DTO\Request\DataSet $request, DTO\Response $response) 
    {
        try {
            $this->service->saveContaCorrente($request->getDataSet()->getRows());
        }
        catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveObservacao(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        $this->service->saveObservacao($row);
    }
    
    public function salvarPrazos(DTO\Request\Row $request, DTO\Response $response)
    {
        $rows = $request->getRow();
        
        foreach ($rows as $row) {
            if ($row["__is_new"]) {
                $this->service->insertPrazo($row); 
            } else {
                $this->service->updatePrazo($row);
            }
        }
    }
    
    public function getTipoBaixa(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        $tipoBaixa = $this->service->getTipoBaixa($row);
    
        $response->addDataSet(new DataSet('res', $tipoBaixa));
    }
    
    public function getCompacArq(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        $value = $this->service->getCompacArq($row);
        $response->addDataSet(new DataSet('rus', $value));
    }
    
    public function salvarCompacArq(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        $this->service->salvarCompacArq($row);
    }
    
    public function getTipoForn(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        $value = $this->service->getTipoForn($row);
        $response->addDataSet(new DataSet('resp', $value));
    }
    
    public function salvarDigitoCST(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        $this->service->salvarDigitoCST($row);
    }
    public function salvarContaPagar(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        $this->service->salvarContaPagar($row);
    }
    
    public function getFornecedorInscricao(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        
        $franqueadora = $this->service->getFranqueadora();
        
        if($franqueadora) {
            $inscricao = str_replace(array("/", "-", "."), "", $row["CDFORNECED"]);
            $fornecedor = $this->service->getFornecedorInscricao($inscricao, $this->environment->getCdfilial());
            $response->addDataSet(new DataSet('fornecedor', $fornecedor));
        }
        else {
            $response->addDataSet(new DataSet('fornecedor', array()));
        }

    }
    
    public function deleteFornecedor(DTO\Request $request, DTO\Response $response) 
    { 
        try {
            $this->service->iniciaTransacao();
            
            $row = $request->getRow();
            $fornecedor = $row['CDFORNECED'];
            
            if(!$this->service->getFranqueadora() || count($this->service->getAllFiliForn($fornecedor)) <= 1){
                //Excluir registros da tabela PARAMFORNE
                $this->dataSourceName = 'paramforne_fornecedor';
                $paramforne = $this->service->getParamForne($row['CDFORNECED']);
                if(is_array($paramforne)){
                    $paramforne[0]["__is_new"] = false;
                    $dataSave = new DataSet('paramforne_fornecedor', (array)$paramforne);
                    $requestSave = new DTO\Request\DataSet($dataSave, $request->getMethod(), $this->dataSourceName, $request->getUserId());
                    $customResponse = new DTO\Response();
                    parent::delete($requestSave, $customResponse);    
                }
                
                //Excluir registros da tabela FORNECEDOR
                $dataSource = substr($request->getRoutePath(), 1, strlen($request->getRoutePath()) - 8);
                $this->dataSourceName = $dataSource;
                parent::delete($request, $response);
            }else{
                $filial = $this->environment->getCdfilial();
                !$this->service->deleteFiliForn($filial, $fornecedor);
            }
            
            $this->service->commit();
        } catch (\Exception $e) {
            $this->service->rollback();
		} 
    }
    
    /**
     * Chama a função service para inativar os fornecedores de acordo com o número de meses recebido.
     *
     * @param  DTO\Request\Row  $request
     * @param  DTO\Response     $reponse
     * @return void
     */
    
    public function inativarFornecedores(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $MESES_SEM_MOVIMENTAÇÃO = $row["MESES_SEM_MOVIMENTACAO"];

        try {
            
            $fornecedoresInativados = $this->service->inativarFornecedores($MESES_SEM_MOVIMENTAÇÃO);
            $dataSet = new DataSet("fornecedoresInativados", array($fornecedoresInativados));
            $response->setDataSets(array($dataSet));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
		}

    }
}