<?php
namespace Planejamento\Logic\Prato;

use \Geral\Logic\Picture;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Geral\Util\PictureProduto;
use Geral\Util\Util;
use Planejamento\Entities\Pratfili;
use Planejamento\Entities\Pratpadr;
use Planejamento\Entities\Pratserv;
use Planejamento\Entities\Recefili;
use Planejamento\Entities\Recepadr;
use \Planejamento\Entities\Prato;
use Planejamento\Entities\Receserv;
use \Planejamento\Entities\Limpepra;
use \Planejamento\Util\Parametros;
use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;
use \Geral\Entities\Produto;
use Planejamento\Repositories\Recepadr as RepositoryRecepadr;
use Planejamento\Repositories\Recefili as RepositoryRecefili;
use Planejamento\Repositories\Receserv as RepositoryReceserv;
use Setforms2\Shared\Registry;


class PratoLogic{
    
    private $tamanhoNivel;
    private $codigoNivelAnterior;
    
    const CDPRATO               = 'CDPRATO';
    const SGUNIDADE             = 'SGUNIDADE';
    const VRARREDTRANS          = 'VRARREDTRANS';
    const CDFILAUXPRAT          = 'CDFILAUXPRAT';
    const FILIALRECEITA         = 'FILIALRECEITA';
    const NMPRATO               = 'NMPRATO';
    const PICTURE               = 'CDPICTPRAT';
    const NRORG                 = 'NRORG';
    const CDNVPRATO             = 'CDNVPRATO';
    const QT_PESSOA             = 'QTPESSOA';
    const PERCENTUAL            = 'REAJUSTEPERC';
    const CDFILAUXPRAT_FILTRO   = 'CDFILIALFILTRO';


    const MULTIPLICAR = 'M';

    const DIVIDIR = 'D';

    const STATE = '_STATE';


    /**
     * Valida o preenchimento dos campos na tela de prato.
     * Revisada após alteração do layout.
     *
     * @param array $registro
     *
     * @throws \Exception
     */
    private static function validaCampos(array $registro){
        $picture = new Picture($registro[self::NRORG], self::PICTURE);
        $numeroNiveis = $picture->getNumeroDeNiveis();
        $codigoValido = self::validaCodigoPrato($registro[self::NRORG], $registro[self::CDPRATO], $registro[self::CDFILAUXPRAT], $registro[self::STATE]);
        $positivo = self::verificaValorPositivo($registro[self::VRARREDTRANS]);

        try{
            if(empty($registro[self::FILIALRECEITA]) && $registro[self::CDNVPRATO] == $numeroNiveis)
                tk_abort(tk_get_message_by_name('msg_pla01300_informarReceita'));
            if(is_string($codigoValido) && $registro['_ID'] <> 1 && $registro['_STATE'] <> 'U')
	            tk_abort(tk_get_message_by_name($codigoValido));
	        if(empty($registro[self::NMPRATO]))
	            tk_abort(tk_get_message_by_name('msg_pla01300_informarNome'));
	        if(empty($registro[self::SGUNIDADE]) && $registro[self::CDNVPRATO] == $numeroNiveis)
	            tk_abort(tk_get_message_by_name('msg_pla01300_informarUnidade'));
            if($positivo == false && $registro[self::CDNVPRATO] == $numeroNiveis)
                tk_abort(tk_get_message_by_name('msg_pla01300_arredondamentoTransPositivo'));
        } catch(\Exception $e){
			throw $e;			        	
        }
        
    }

    public static function verificaValorPositivo($valor){
        if($valor >= 0){
            return true;
        } else{
            return false;
        }
    }



    /**
     * Verifica qual o �ltimo n�vel da �rvore e compara com o n�vel do prato
     * Revisado após alteração do layout da tela de Prato
     *
     * @param        $codigoPrato
     * @param        $nrorg
     * @param string $cdfilauxprat
     */
    public static function controlaFields($codigoPrato, $nrorg, $cdfilauxprat = 'P'){
        $objPrato = new self();
        $picture = new Picture($nrorg, self::PICTURE);

        $numeroNiveis = $picture->getNumeroDeNiveis();
        $nivelCodigo = $objPrato->codigoNivelPrato($nrorg, $codigoPrato);
        
        if($nivelCodigo == $numeroNiveis){
            tk_enable_field (self::SGUNIDADE);
            tk_enable_field (self::VRARREDTRANS);
            tk_set_visible_field(self::FILIALRECEITA);
            
//            tk_set_enable_action('TK_NEW_ROW_U', 'tab_pla01300_000_001', 'win_pla01300_000');
//            tk_set_enable_action('TK_DELETE_ROW_U', 'tab_pla01300_000_001', 'win_pla01300_000');
//            tk_set_enable_action('TK_OPEN_SELECTION', 'tab_pla01300_000_001', 'win_pla01300_000');
//            tk_set_enable_action('TK_RELOAD_FILTER', 'tab_pla01300_000_001', 'win_pla01300_000');
//			tk_set_visible_field('BUTTON_RECEITA', 'tab_pla01300_000_002','win_pla01300_000');
            tk_enable_tab('pla01300_000_001');
            tk_enable_tab('pla01300_000_003');
            tk_enable_tab('pla01300_000_005');
            tk_enable_tab('pla01300_000_007');
            tk_set_field_required(self::SGUNIDADE);
        }
        else{
            tk_disable_field(self::SGUNIDADE);
            tk_disable_field(self::VRARREDTRANS);
            tk_set_invisible_field(self::FILIALRECEITA);       
            
//            tk_set_disable_action('TK_NEW_ROW_U', 'tab_pla01300_000_001', 'win_pla01300_000');
//            tk_set_disable_action('TK_DELETE_ROW_U', 'tab_pla01300_000_001', 'win_pla01300_000');
//            tk_set_disable_action('TK_OPEN_SELECTION', 'tab_pla01300_000_001', 'win_pla01300_000');
//            tk_set_disable_action('TK_RELOAD_FILTER', 'tab_pla01300_000_001', 'win_pla01300_000');
//            tk_set_invisible_field('BUTTON_RECEITA','tab_pla01300_000_002','win_pla01300_000');
            tk_disable_tab('pla01300_000_001');
            tk_disable_tab('pla01300_000_003');
            tk_disable_tab('pla01300_000_005');
            tk_disable_tab('pla01300_000_007');
            tk_set_field_not_required(self::SGUNIDADE);
        }
        
    }


    /**
     * Sugere novo c�digo para o prato, de acordo com o n�vel em que est� sendo
     * inserido na �rvore.
     */
    public static function novoCodigoPrato($codigoPrato, $cdfilauxprat, $cdnvprato, $nrorg){
        $objPrato = new self();
        $picture = new Picture($nrorg, self::PICTURE);
        $numeroNiveis = $picture->getNumeroDeNiveis();
        $nivelPratoOriginal = $cdnvprato;
        
        //Verifica qual pode ser o primeiro novo c�digo de acordo com o c�digo da �rvore e o n�vel.
        if($cdnvprato == $numeroNiveis){
            $codigoNivelAnterior = $objPrato->codigoNivelAnterior($nrorg, $codigoPrato);
            $novoCodigo = substr($codigoPrato, strlen($codigoNivelAnterior));
            $codigoPrato = $codigoNivelAnterior;
            $nivelCompleto = true;
        } else{
            if($cdnvprato == $numeroNiveis - 1)
                $novoCodigo = 0;
            else
                $novoCodigo = 1;
            $cdnvprato++;
            $nivelCompleto = false;
        }
        
        //Consulta os codigo j� existentes para o novo n�vel
        /** @var $prato Prato */
        $codigos = Prato::getObjectsByWhere(
                "CDPRATO like ?
                 AND CDFILAUXPRAT = ?
                 AND CDNVPRATO = ?",
                array($codigoPrato."%", $cdfilauxprat, $cdnvprato),
                "CDPRATO"
        );
        
        if(count($codigos) == 0){
            $mascara = $picture->getMascara();
            $mascaraArray = explode('.', $mascara);
            $codigoExistente = $mascaraArray[$nivelPratoOriginal];
        } else{
            //Verifica a posicao do array em que sera realizada a busca por um novo codigo valido.
            if($nivelCompleto == true){
                foreach($codigos as $indice=>$codigo){
                    $cdprato = $codigo->getCdprato();
                    $codigoExistente = substr($cdprato, strlen($codigoPrato));

                    if($novoCodigo == $codigoExistente){
                        $posicao = $indice;
                        break;
                    }
                }
            } else
                $posicao = 0;

            //Busca o menor c�digo v�lido.
            for($i = $posicao; $i < count($codigos); $i++){
                $cdprato = $codigos[$i]->getCdprato();
                $codigoExistente = substr($cdprato, strlen($codigoPrato));

                if($novoCodigo < $codigoExistente){
                    break;
                }
                $novoCodigo++;
            }
        }
        
        $novoCodigo = str_pad($novoCodigo, strlen($codigoExistente), "0",STR_PAD_LEFT);
        return $codigoPrato.$novoCodigo;
        
    }

    public static function codigoNivelAnteriorPratoProd(){
        $picturePrato = new Picture(Ambiente::getNrorg(), self::PICTURE);

        //Pega mascara utilizada na arvore
        $mascara = $picturePrato->getMascara();
        $nivel = explode('.',$mascara);

        //Monta o codigo do primeiro nivel do prato de produto
        $cdprato = '';
        $cdprato = str_pad($cdprato,strlen($nivel[0]),"P");

        return $cdprato;
    }


    /**
     * Insere novo prato criado a partir do produto.
     */
    public static function novoPratoProduto(array $registro){
        $picturePrato = new Picture(Ambiente::getNrorg(), self::PICTURE);
        $mascara = $picturePrato->getMascara();

        $cdprato = self::codigoNivelAnteriorPratoProd();
        $cdPratoPai = $cdprato;

        //Verifica se existe prato de produto pai
        $pratoPai = Prato::getObjectsByWhere(
            "CDPRATO = ? AND CDFILAUXPRAT = ? AND NRORG = ?",
            array($cdprato,'P',$registro['NRORG']),
            "CDPRATO"
        );

        //Caso nao exista, insere o resgistro pai do prato de produto
        if (empty($pratoPai)) {
            $novoPratoPai = new Prato();

            $registroPai['CDPRATO'] = $cdprato;
            $registroPai['NMPRATO'] = tk_get_message_by_name('msg_pla01300_pratoDeProduto');
            $registroPai['CDFILAUXPRAT'] = $registro['CDFILAUXPRAT'];
            $registroPai['CDFILIAL'] = $registro['CDFILIAL'];
            $registroPai['CDNVPRATO'] = '1';
            $registroPai['CDPRATONIVANTE'] = '';
            $registroPai['SGUNIDADE'] = '';
            $registroPai['VRARREDTRANS'] = '';
            $registroPai['IDATIVO'] = 'S';
            $registroPai['NRORG'] = $registro['NRORG'];
            $registroPai['CDOPERINCLUSAO'] = $registro['CDOPERINCLUSAO'];
            $registroPai['DTINCLUSAO'] = $registro['DTINCLUSAO'];
            $registroPai['DTULTATU'] = $registro['DTULTATU'];
            $registroPai['NRORGINCLUSAO'] = $registro['NRORGINCLUSAO'];

            $novoPratoPai->inserePrato($registroPai);
            \Setforms2\Shared\Registry::getEntityManager()->flush();

            $cdprato = str_pad($cdprato, strlen(str_replace('.', '', $mascara)), '0');
        } else {
            //Caso exista, verifica se ja possui prato de produto
            $pratoFilho = Prato::getObjectsByWhere(
                "CDPRATO LIKE ? AND CDFILAUXPRAT = ? AND CDPRATONIVANTE = ? AND NRORG = ?",
                array($cdprato.'%','P',$cdPratoPai,$registro['NRORG']),
                "CDPRATO DESC"
            );
            if(empty($pratoFilho)){
                $cdprato = str_pad($cdprato, strlen(str_replace('.', '', $mascara)), '0');
            } else{
                $cdprato = $pratoFilho[0]->getCdprato();
                $cdprato++;
            }
        }

        //Insere o prato de produto com novo codigo
        $novoPratoFilho = new Prato();
        $registro['CDPRATO'] = $cdprato;
        $registro['CDPRATONIVANTE'] = $cdPratoPai;

        \Planejamento\Repositories\Prato::disableTrigger();

        $novoPratoFilho->inserePrato($registro);
        \Setforms2\Shared\Registry::getEntityManager()->flush();

        \Planejamento\Repositories\Prato::enableTrigger();

        return $cdprato;
    }

    public static function novaReceitaPratoProduto($cdprato,$cdfilauxprat,$quantidade,$cdproduto,$idtpreqprodpadr,$idutcompentdirp,$ativo,$nrorg,$dtinclusao,$dtultatu,$cdoperador){
        $receitaPratoProduto = new Recepadr();

        $receitaPratoProduto->insereReceitaPadrao($cdprato,$cdfilauxprat,$quantidade,$cdproduto,$idtpreqprodpadr,$idutcompentdirp,$ativo,$nrorg,$dtinclusao,$dtultatu,$cdoperador);
        \Setforms2\Shared\Registry::getEntityManager()->flush();

    }

    public static function atualizaNmpratoProd($nmProduto,$nmPratoProd,$cdPratoProd,$nrorg){
        if($nmProduto != $nmPratoProd && $nmPratoProd != ''){
            /** @var $prato $prato */
            $prato = Prato::getObjectByWhere(
                "CDPRATO = ? AND CDFILAUXPRAT = ? AND NRORG = ?",
                array($cdPratoProd, 'P', $nrorg)
            );
            $prato->setNmprato($nmProduto);
            $prato->persist();

            \Setforms2\Shared\Registry::getEntityManager()->flush();
        }
    }

    
    /**
     * Recebe PK da tabela PRATO mais o nrorg, e faz a valida��o do c�digo do prato
     * informado pelo usu�rio
     */
    public static function validaCodigoPrato($nrorg, $codigoPrato, $cdfilauxprat, $state){
        $objPrato = new self();
        $picture = new Picture($nrorg, self::PICTURE);
        
        $tamanhoNivelAcumulado = $picture->getTamanhoNivelAcumulado();
        $nivelPrato = $objPrato->codigoNivelPrato($nrorg, $codigoPrato);
        $nivelAterior = $objPrato->codigoNivelAnterior($nrorg, $codigoPrato);
        $objPrato->controlaFields($codigoPrato, $nrorg, $cdfilauxprat);
        
        if(empty($codigoPrato)){
            $mensagem = 'msg_pla01300_informarCodigo';
            return $mensagem;
        } else{
            if(strlen($codigoPrato) <> $tamanhoNivelAcumulado[$nivelPrato-1]){
                $mensagem = 'msg_pla01300_codigoPratoPreenchidoNivelIncorreto';
                return $mensagem;
            } else{
                /** @var $prato Prato */
                $prato = Prato::getObjectByWhere(
                        "CDPRATO = ?
                         AND NRORG = ?",
                        array($nivelAterior, $nrorg)
                );

                if(is_object($prato) == false && $nivelPrato <> 1){
                    $mensagem = 'msg_pla01300_nivelAnteriorNaoCadastrado';
                    return $mensagem;
                } else{
                    if ($state != 'U') {
                    /** @var $prato Prato */
                    $pratoAtual = Prato::getObjectByWhere(
                            "CDPRATO = ?
                             AND CDFILAUXPRAT = ?
                             AND NRORG = ?",
                            array($codigoPrato, $cdfilauxprat, $nrorg)
                    );

                    if(is_object($pratoAtual)){
                        $mensagem = 'msg_pla01300_codigoPratoPreenchidoExistente';
                        return $mensagem;
                    } else{
                        $mensagem = self::paramBloqPratoPadraoMesmoCodPratoLocal($codigoPrato,$nrorg);
                        return $mensagem;
                    }
                }
            }
        }
        }

    }


    /**
     * Recebe o nrorg e o c�digo do prato digitado pelo usu�rio e calcula o
     * tamanho do n�vel e o c�digo do n�vel anterior.
     */
    private function tratamentoCodigo($nrorg, $codigoPrato){
        $this->tamanhoNivel = 1;
        $auxNivelAnterior = "";
                     
        $picture = new Picture($nrorg, self::PICTURE);
        $codigoComMascara = $picture->getCodigoComMascara($codigoPrato);
            
        for($i = 0; $i < strlen($codigoComMascara); $i++){
            if($codigoComMascara[$i] <> "."){
                $auxNivelAnterior .= $codigoComMascara[$i];
            }
            else{
                $this->codigoNivelAnterior = $auxNivelAnterior;
                $this->tamanhoNivel++;
            }
        }
        
    }
    
    
    //Recebe o nrorg e o c�digo informado pelo usu�rio e retorna o tamanho do n�vel.
    public function codigoNivelPrato($nrorg, $codigoPrato){
        $this->tratamentoCodigo($nrorg, $codigoPrato);
        
        return $this->tamanhoNivel;
    }
        
    
    //Recebe o nrorg e o c�digo informado pelo usu�rio e retorna o c�digo do n�vel anterior.
    public function codigoNivelAnterior($nrorg, $codigoPrato){
        $this->tratamentoCodigo($nrorg, $codigoPrato);
        return $this->codigoNivelAnterior;
    }
    
    
    //Verifica se � uma inser��o ou uma atualiza��o
    public static function insereAtualizaPrato(array $registro){
    	try {
        	self::validaCampos($registro);
    	} catch(\Exception $e){
    		tk_message($e->getMessage(),'error');
    		return;
    	}
        
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try{
            if($registro['_STATE'] == 'U'){
//            if($registro['_ID'] == 1 && $registro['_STATE'] == 'U'){
                /** @var $prato Prato */
                $prato = Prato::getObjectByWhere(
                    "CDPRATO = ?
                     AND CDFILAUXPRAT = ?
                     AND NRORG = ?",
                    array($registro[self::CDPRATO], $registro[self::CDFILAUXPRAT], $registro[self::NRORG])
                );

                $nomeAlterado = self::verificaAlteracaoNmprato($prato->getNmprato(),$registro[self::NMPRATO]);
                $prato->atualizaPrato($registro);
                
//                if($nomeAlterado){
//                    tk_reload_data_with_filter('tab_pla01300_000_000', 'win_pla01300_000');
//                }
            } else{
                $prato = new Prato();
                $prato->inserePrato($registro);
//                tk_reload_data_with_filter('tab_pla01300_000_000', 'win_pla01300_000');
            }

            $connection->flush();
            $connection->commit();
            tk_message(tk_get_message_by_name('msg_salvo'), 'success');
//            tk_reload_data_with_filter();
//            tk_cancel();
        } catch (\Exception $e) {
            tk_warning($e->getMessage());
            $connection->rollback();
        }
    }
    
    
    public static function deletaPrato($cdprato, $cdfilauxprat, $nrorg){
        /** @var $prato Prato */
        $prato = Prato::getObjectByWhere(
                "CDPRATONIVANTE = ?",
                array($cdprato)
        );
        
        if(is_object($prato)){
        	try{
            	tk_abort(tk_get_message_by_name('msg_pla01300_exclusaoNaoPermitida'));
        	} catch(\Exception $e){
        		throw $e;
        	}
        } else{
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();
            try{
                //Chama procedure para realizar exclusões nas tabelas
                self::procedureDelCascadePrato($cdprato,$cdfilauxprat);

                /** @var $prato Prato */
                $prato = Prato::getObjectByWhere(
                        "CDPRATO = ?
                        AND CDFILAUXPRAT = ?
                        AND NRORG = ?",
                        array($cdprato, $cdfilauxprat, $nrorg)
                );

                $prato->delete();
                $connection->flush();
                $connection->commit();

//                tk_reload_data_with_filter('tab_pla01300_000_000', 'win_pla01300_000');
            } catch (\Exception $e) {
                $connection->rollback();
                throw $e;
            }
        }
    }
    
    
    public static function setCdfilauxprat($filialreceita){
        if($filialreceita == 'P')
            return 'P';
        else{
            $filial = tk_get_context_vars('CDFILAUXPRAT_FILTRO', tk_get_active_window_name());
            return $filial;
        }
        
    }
    
    
    public static function abrirConsultaNutriente($cdprato, $cdfilauxprat, $filiproc, $cdfilcentpme, $cdtipocard, $cdfilial, $cdservfili){
    	if(empty($cdservfili))
    		$cdservfili = 'X';
    	
        $sql = \Windows\WinPla01304000\TabPla01304000000\Query::consultaNutrientes($cdprato, $cdfilauxprat, $filiproc, $cdfilcentpme, $cdtipocard, $cdfilial, $cdservfili);
        
        return $sql;
    }
    
    
    public static function controlaActionReceita($cdfilauxprat){
    	if(is_numeric($cdfilauxprat)){
    		tk_set_disable_action('TK_NEW_ROW_U', 'pla01300_000_003', 'win_pla01300_000');
            tk_set_disable_action('TK_DELETE_ROW_U', 'pla01300_000_003', 'win_pla01300_000');
            tk_set_disable_action('act_importarReceita', 'pla01300_000_003', 'win_pla01300_000');
            tk_set_disable_action('act_AbrirConsultaNutriente','pla01300_000_003','win_pla01300_000');
            tk_disable_field('CDFILIAL','pla01300_000_005_filter','win_pla01300_000');
			tk_disable_field('CDFILIAL_FILTER','pla01300_000_007_filter','win_pla01300_000');

            tk_set_disable_action('TK_NEW_ROW_U', 'pla01300_000_004', 'win_pla01300_000');
            tk_set_disable_action('TK_DELETE_ROW_U', 'pla01300_000_004', 'win_pla01300_000');
    	} else{
    		tk_set_enable_action('TK_NEW_ROW_U', 'pla01300_000_003', 'win_pla01300_000');
    		tk_set_enable_action('TK_DELETE_ROW_U', 'pla01300_000_003', 'win_pla01300_000');
    		tk_set_enable_action('act_importarReceita', 'pla01300_000_003', 'win_pla01300_000');
    		tk_set_enable_action('act_AbrirConsultaNutriente','pla01300_000_003','win_pla01300_000');
    		tk_enable_field('CDFILIAL','pla01300_000_005','win_pla01300_000');
    		tk_enable_field('CDFILIAL','pla01300_000_007','win_pla01300_000');

            tk_set_enable_action('TK_NEW_ROW_U', 'pla01300_000_004', 'win_pla01300_000');
            tk_set_enable_action('TK_DELETE_ROW_U', 'pla01300_000_004', 'win_pla01300_000');
    	}
    	
    }
    
    
    public static function calculaNutriente($aprovNutri,$qntCalcNutri){
		$novoCalcNutri = $aprovNutri * $qntCalcNutri / 100;
    	return $novoCalcNutri;
    	
    }

    /**
     * Encontra o fator de conversão com base na quantidade requerida e na quantidade nutriente
     *
     * @param $qtdeLiq
     * @param $qtdeNutri
     * @param $aprovNutri
     * @param $quantidadeReq
     * @param $quantidadeBru
     * @return float
     */
    public static function encontraFatorConversaoUnidade($qtdeLiq,$qtdeNutri,$aprovNutri,$quantidadeReq,$quantidadeBru){
        $paramQuantidade = Parametros::getInstancia(tk_get_env_var(self::CDFILAUXPRAT_FILTRO))->getIdLiqBrut();
        if($paramQuantidade == 'L') {
            $qtdeNutriOri = 100 * $qtdeNutri / $aprovNutri;
            $fatorConv = $qtdeNutriOri / $qtdeLiq;
        } else{
            $fatorConv = $quantidadeReq / $quantidadeBru;
        }

        if($fatorConv == 0){
            $fatorConv = 1;
        }

        return $fatorConv;
    }


    public static function paramBloqPratoPadraoMesmoCodPratoLocal($cdprato, $nrorg)
    {
        $param = Parametros::getIdBloqPraDup();

        if ($param == 'S') {
            $prato = Prato::getObjectByWhere(
                "CDPRATO = ?
                AND NRORG = ?",
                array($cdprato, $nrorg)
            );

            if (is_object($prato)) {
                $mensagem = 'msg_pla01300_codigoPratoPreenchidoExistenteFilial';
                return $mensagem;
            } else {
                return true;
            }
        } else {
            return true;
        }

    }

    /**
     * Chama procedure 'DEL_CASCADE_PRATO' ao excluir algum prato.
     *
     * @param $cdprato
     * @param $cdfilauxprat
     *
     * @return null
     */
    public static function procedureDelCascadePrato($cdprato,$cdfilauxprat){

        $procedure = new Procedure('DEL_CASCADE_PRATO');
        $procedure->addParam(
            new Param (
                'P_CDFILAUXPRAT',
                Param::PARAM_INPUT,
                $cdfilauxprat,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param (
                'P_CDPRATO',
                Param::PARAM_INPUT,
                $cdprato,
                Param::PARAM_TYPE_STR
            )
        );

        $procedure->call();
    }

    public static function verificaAlteracaoNmprato($nomeAntes, $nomeDepois){
        if($nomeAntes == $nomeDepois){
            return false;
        } else{
            return true;
        }
    }

    /**
     * Ajusta as quantidades na receita padrão
     *
     * @param String $nrorg        Organização
     * @param String $cdprato      Prato
     * @param String $cdfilauxprat Fila Aux Prato
     * @param String $valor        Valor
     * @param String $operacao     Operação
     *
     * @throws \Exception
     *
     * @return null
     */
    public static function ajustaQuantidadesReceitaPadrao(
        $nrorg, $cdprato,
        $cdfilauxprat, $valor, $operacao
    )  {
        try {
            /** @var \Planejamento\Entities\Recepadr $receitaPadrao */
            $receitaPadrao = Recepadr::getProdutosReceitaPadrao(
                $nrorg, $cdprato, $cdfilauxprat
            );

            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            foreach ($receitaPadrao as $receita) {
                if ($operacao == self::MULTIPLICAR) {
                    /** @var \Planejamento\Entities\Recepadr $receita */
                    $qtbrutapr = $receita->getQtbrutapr() * $valor;
                    $qtbrutaprunr = $receita->getQtbrutaprunr() * $valor;
                    $qtcalcnutrpr = $receita->getQtcalcnutrpr() * $valor;
                    $qtcomplprodpadr = $receita->getQtcomplprodpadr() * $valor;
                    $qtprodprat = $receita->getQtprodprat() * $valor;
                    $qtunidestpr = $receita->getQtunidestpr() * $valor;


                } elseif ($operacao == self::DIVIDIR) {
                    /** @var \Planejamento\Entities\Recepadr $receita */
                    $qtbrutapr = $receita->getQtbrutapr() / $valor;
                    $qtbrutaprunr = $receita->getQtbrutaprunr() / $valor;
                    $qtcalcnutrpr = $receita->getQtcalcnutrpr() / $valor;
                    $qtcomplprodpadr = $receita->getQtcomplprodpadr() / $valor;
                    $qtprodprat = $receita->getQtprodprat() / $valor;
                    $qtunidestpr = $receita->getQtunidestpr() / $valor;
                }

                Recepadr::atualizaQtdeReceitaPadrao(
                    $receita, $qtprodprat, $qtbrutapr, $qtbrutaprunr,
                    $qtunidestpr, $qtcalcnutrpr, $qtcomplprodpadr
                );

                $connection->flush();
            }

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    /**
     * Ajusta as quantidades na receita local
     *
     * @param String $nrorg        Organização
     * @param String $cdprato      Prato
     * @param String $cdfilauxprat Fila Aux Prato
     * @param String $valor        Valor
     * @param String $operacao     Operação
     *
     * @throws \Exception
     *
     * @return null
     */
    public static function ajustaQuantidadesReceitaLocal(
        $nrorg, $cdfilial, $cdprato,
        $cdfilauxprat, $valor, $operacao
    )  {
        try {
            /** @var \Planejamento\Entities\Recefili $receitaLocal */
            $receitaLocal = Recefili::getProdutosReceitaLocal(
                $nrorg, $cdfilial, $cdprato, $cdfilauxprat
            );

            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            foreach ($receitaLocal as $receita) {
                if ($operacao == self::MULTIPLICAR) {
                    /** @var \Planejamento\Entities\Recefili $receita */
                    $qtbrutalo = $receita->getQtbrutalo() * $valor;
                    $qtbrutalounr = $receita->getQtbrutalounr() * $valor;
                    $qtcalcnutrlo = $receita->getQtcalcnutrlo() * $valor;
                    $qtcomplprodfili = $receita->getQtcomplprodfili() * $valor;
                    $qtprodrecloc = $receita->getQtprodrecloc() * $valor;
                    $qtunidestlo = $receita->getQtunidestlo() * $valor;

                } elseif ($operacao == self::DIVIDIR) {
                    /** @var \Planejamento\Entities\Recefili $receita */
                    $qtbrutalo = $receita->getQtbrutalo() / $valor;
                    $qtbrutalounr = $receita->getQtbrutalounr() / $valor;
                    $qtcalcnutrlo = $receita->getQtcalcnutrlo() / $valor;
                    $qtcomplprodfili = $receita->getQtcomplprodfili() / $valor;
                    $qtprodrecloc = $receita->getQtprodrecloc() / $valor;
                    $qtunidestlo = $receita->getQtunidestlo() / $valor;
                }

                Recefili::atualizaQtdeReceitaLocal(
                    $receita, $qtbrutalo, $qtbrutalounr,
                    $qtcalcnutrlo, $qtcomplprodfili, $qtprodrecloc, $qtunidestlo
                );

                $connection->flush();
            }

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }
    /**
     * Ajusta as quantidades na receita por serviço
     *
     * @param String $nrorg        Organização
     * @param String $cdprato      Prato
     * @param String $cdfilauxprat Fila Aux Prato
     * @param String $valor        Valor
     * @param String $operacao     Operação
     *
     * @throws \Exception
     *
     * @return null
     */
    public static function ajustaQuantidadesReceitaServico(
        $nrorg, $cdprato, $servico, $filial,
        $cdfilauxprat, $valor, $operacao
    )  {
        try {
            /** @var \Planejamento\Entities\Receserv $receitaServico */
            $receitaServico = Receserv::getProdutosReceitaServico(
                $nrorg, $cdprato, $cdfilauxprat, $servico, $filial
            );

            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            foreach ($receitaServico as $receita) {
                if ($operacao == self::MULTIPLICAR) {
                    /** @var \Planejamento\Entities\Receserv $receita */
                    $qtbrutase = $receita->getQtbrutase() * $valor;
                    $qtbrutaseunr = $receita->getQtbrutaseunr() * $valor;
                    $qtcalcnutrse = $receita->getQtcalcnutrse() * $valor;
                    $qtcomplprodserv = $receita->getQtcomplprodserv() * $valor;
                    $qtprodrecser = $receita->getQtprodrecser() * $valor;
                    $qtunidestse = $receita->getQtunidestse() * $valor;


                } elseif ($operacao == self::DIVIDIR) {
                    /** @var \Planejamento\Entities\Receserv $receita */
                    $qtbrutase = $receita->getQtbrutase() / $valor;
                    $qtbrutaseunr = $receita->getQtbrutaseunr() / $valor;
                    $qtcalcnutrse = $receita->getQtcalcnutrse() / $valor;
                    $qtcomplprodserv = $receita->getQtcomplprodserv() / $valor;
                    $qtprodrecser = $receita->getQtprodrecser() / $valor;
                    $qtunidestse = $receita->getQtunidestse() / $valor;
                }

                Receserv::atualizaQtdeReceitaServico(
                    $receita, $qtbrutase, $qtbrutaseunr,
                    $qtcalcnutrse, $qtcomplprodserv, $qtprodrecser, $qtunidestse
                );

                $connection->flush();
            }

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public static function atualizaFatorCorrecao($nrorg, $filial, $prato, $filAux, $produto, $idLiqBruto)
    {

        $maxNivelProd = PictureProduto::retornaNumeroMaximoDeNiveis();
        $resultado = RepositoryRecepadr::verificaProdInativoFilial($nrorg, $filial, $prato, $filAux, $maxNivelProd);


        foreach ($resultado as $produtoinativo) {
            if ($produtoinativo['CDARVPROD'] == $produto) {
                if ($produtoinativo['IDPRODATIV'] == 'I' || $produtoinativo['IDPRODATIVPR'] == 'N') {
                    $mensagem = tk_get_message_by_name('msg_produtoInativo');
                    tk_message($mensagem);
                    return false;
                } else {
                    /** @var $prod Produto */
                    $prod = Produto::getObjectByWhere('CDPRODUTO = ? and NRORG = ?', array($produto, $nrorg));
                    $vrpefatocorr        =$prod->getVrpefatocorr();
                    /** @var $registro Recepadr */
                    $registro = Recepadr::getObjectByWhere('CDPRODUTO = ? and NRORG = ? and CDPRATO = ?', array($produto, $nrorg, $prato));

                    $ganhococao       = $registro->getVrpefatcocpr();
                    $aprovNutricional = $registro->getVrpefatnutpr();
                    $perdalimpeza     = $registro->getVrpefatcorpr();
                    $quantidadeL      = $registro->getQtprodprat();
                    $quantidadeB      = $registro->getQtbrutaprunr();
                    $quantidade       = '';


                    if ($idLiqBruto == 'B') {
                        $quantidade = $quantidadeL + (($quantidadeL * $ganhococao)  / 100);

                    } else if ($idLiqBruto == 'L') {
                        $quantidade = $quantidadeB + (($quantidadeB * $ganhococao)  / 100);
                    }

                    if ($perdalimpeza == $vrpefatocorr) {
                        $novoValor = $quantidade - (($vrpefatocorr * $quantidade) / 100);
                        $quantidade = $novoValor;
                        $quanValorNutri = $aprovNutricional * $quantidade / 100;
                        $vrpefatcopr = $vrpefatocorr;
                    } else {
                        $vrpefatcopr = $vrpefatocorr;
                        $novoValor = $quantidade - (($vrpefatocorr * $quantidade) / 100);
                        $quantidade = $novoValor;
                        $quanValorNutri = $aprovNutricional * $quantidade / 100;
                    }

                    if ($idLiqBruto == 'L') {
                        RepositoryRecepadr::atualizaProdFatorCorrecaoLiquida($produto, $quantidade, $prato, $nrorg, $filAux, $quanValorNutri, $vrpefatcopr);

                    } else if ($idLiqBruto == 'B') {
                        RepositoryRecepadr::atualizaProdFatorCorrecaoBruta($produto, $quantidade, $prato, $nrorg, $filAux, $quanValorNutri, $vrpefatcopr);

                    }
                }
                return true;
            }

        }
    }



    public static function atualizaTodosProdutos($nrorg, $cdprato, $filial, $idLiqBrut)
    {

        /** @var $registro Recepadr */
        $registro = Recepadr::getObjectsByWhere('CDPRATO = ? AND NRORG = ?', array($cdprato, $nrorg));
        /** @var $linha Recepadr */
        foreach ($registro as $linha) {
            $filAux           = $linha->getCdfilauxprat();
            $prato            = $linha->getCdprato();
            $produto          = $linha->getCdproduto();

            $validacao = self::atualizaFatorCorrecao($nrorg, $filial, $prato, $filAux, $produto, $idLiqBrut);

        }
        if ($validacao == true) {
            $msgsucesso = tk_get_message_by_name('msg_importacaoSucesso');
            tk_message($msgsucesso);
        }

    }

    public static function atualizaFatorCorrecaoLocal($nrorg, $filial, $prato, $filAux, $produto, $idLiqBruto)
    {

        $maxNivelProd = PictureProduto::retornaNumeroMaximoDeNiveis();

        $resultado = RepositoryRecefili::verificaProdInativoFilial($nrorg, $filial, $prato, $filAux, $maxNivelProd);


        foreach ($resultado as $produtoinativo) {
            if ($produtoinativo['CDARVPROD'] == $produto) {
                if ($produtoinativo['IDPRODATIV'] == 'I' || $produtoinativo['IDPRODATIVPR'] == 'N') {
                    $mensagem = tk_get_message_by_name('msg_produtoInativo');
                    tk_message($mensagem);
                    return false;
                } else {

                    /** @var $prod Produto */
                    $prod = Produto::getObjectByWhere('CDPRODUTO = ? and NRORG = ?', array($produto, $nrorg));
                    $vrpefatocorr = $prod->getVrpefatocorr();

                    /** @var $registro Recefili */
                    $registro = Recefili::getObjectByWhere('CDPRODUTO = ? and NRORG = ? and CDFILIAL = ? and CDPRATO = ?', array($produto, $nrorg, $filial, $prato));

                    $ganhococao       = $registro->getVrpefatcoclo();
                    $aprovNutricional = $registro->getVrpefatnutlo();
                    $perdalimpeza     = $registro->getVrpefatcorlo();
                    $quantidadeL      = $registro->getQtprodrecloc();
                    $quantidadeB      = $registro->getQtbrutalounr();
                    $quantidade       = '';

                    if ($idLiqBruto == 'L') {
                        $quantidade = $quantidadeB + (($quantidadeB * $ganhococao)  / 100);

                    } else if ($idLiqBruto == 'B') {
                        $quantidade = $quantidadeL + (($quantidadeL * $ganhococao)  / 100);
                    }



                    if ($perdalimpeza == $vrpefatocorr) {
                        $novoValor = $quantidade - (($vrpefatocorr * $quantidade) / 100);
                        $quantidade = $novoValor;
                        $quanValorNutri = $aprovNutricional * $quantidade / 100;
                        $vrpefatcorlo = $vrpefatocorr;
                    } else {
                        $vrpefatcorlo = $vrpefatocorr;
                        $novoValor = $quantidade - (($vrpefatocorr * $quantidade) / 100);
                        $quantidade = $novoValor;
                        $quanValorNutri = $aprovNutricional * $quantidade / 100;
                    }
                    if ($idLiqBruto == 'L') {
                        RepositoryRecefili::atualizaProdFatorCorrecaoLiquidaLocal($produto, $quantidade, $prato, $nrorg, $filAux, $quanValorNutri, $vrpefatcorlo);

                    } else if ($idLiqBruto == 'B') {
                        RepositoryRecefili::atualizaProdFatorCorrecaoBrutaLocal($produto, $quantidade, $prato, $nrorg, $filAux, $quanValorNutri, $vrpefatcorlo);

                    }
                }
                return true;
            }

        }

    }

    public static function atualizaTodosProdutosLocal($nrorg, $cdprato, $filial, $idLiqBrut)
    {

        /** @var $registro Recefili */
        $registro = Recefili::getObjectsByWhere('CDPRATO = ? AND NRORG = ?', array($cdprato, $nrorg));
        /** @var $linha Recefili */
        foreach ($registro as $linha) {
            $filAux           = $linha->getCdfilauxprat();
            $prato            = $linha->getCdprato();
            $produto          = $linha->getCdproduto();

            $validacao = self::atualizaFatorCorrecaoLocal($nrorg, $filial, $prato, $filAux, $produto, $idLiqBrut);

        }
        if ($validacao == true) {
            $msgsucesso = tk_get_message_by_name('msg_importacaoSucesso');
            tk_message($msgsucesso);
        }

    }

    public static function atualizaFatorCorrecaoServico($nrorg, $filial, $prato, $filAux, $produto,$idLiqBruto, $servico)
    {

        $maxNivelProd = PictureProduto::retornaNumeroMaximoDeNiveis();
        $resultado = RepositoryReceserv::verificaProdInativoFilial($nrorg, $filial, $prato, $filAux, $maxNivelProd, $servico);


        foreach ($resultado as $produtoinativo) {
            if ($produtoinativo['CDARVPROD'] == $produto) {
                if ($produtoinativo['IDPRODATIV'] == 'I' || $produtoinativo['IDPRODATIVPR'] == 'N') {
                    $mensagem = tk_get_message_by_name('msg_produtoInativo');
                    tk_message($mensagem);
                    return false;
                } else {
                    /** @var $prod Produto */
                    $prod = Produto::getObjectByWhere('CDPRODUTO = ? and NRORG = ?', array($produto, $nrorg));
                    $vrpefatocorr = $prod->getVrpefatocorr();


                    /** @var $registro Receserv */
                    $registro = Receserv::getObjectByWhere('CDPRODUTO = ? and NRORG = ? and CDFILIAL = ? and CDSERVFILI = ? and CDPRATO = ?'
                        , array($produto, $nrorg, $filial, $servico, $prato));

                    $ganhococao       = $registro->getVrpefatcocse();
                    $aprovNutricional = $registro->getVrpefatnutse();
                    $perdalimpeza     = $registro->getVrpefatcorse();
                    $quantidadeL      = $registro->getQtprodrecser();
                    $quantidadeB      = $registro->getQtbrutaseunr();
                    $quantidade       = '';

                    if ($idLiqBruto == 'L') {
                        $quantidade = $quantidadeB + (($quantidadeB * $ganhococao)  / 100);
                    } else if ($idLiqBruto == 'B') {
                        $quantidade = $quantidadeL + (($quantidadeL * $ganhococao)  / 100);
                    }


                    if ($perdalimpeza == $vrpefatocorr) {
                        $novoValor = $quantidade - (($vrpefatocorr * $quantidade) / 100);
                        $quantidade = $novoValor;
                        $quanValorNutri = $aprovNutricional * $quantidade / 100;
                        $vrpefatcorse = $vrpefatocorr;
                    } else {
                        $vrpefatcorse = $vrpefatocorr;
                        $novoValor = $quantidade - (($vrpefatocorr * $quantidade) / 100);
                        $quantidade = $novoValor;
                        $quanValorNutri = $aprovNutricional * $quantidade / 100;
                    }
                    if ($idLiqBruto == 'L') {
                        RepositoryReceserv::atualizaProdFatorCorrecaoLiquidaServico($produto, $quantidade, $prato, $nrorg, $filAux, $quanValorNutri, $vrpefatcorse);

                    } else if ($idLiqBruto == 'B') {
                        RepositoryReceserv::atualizaProdFatorCorrecaoBrutaServico($produto, $quantidade, $prato, $nrorg, $filAux, $quanValorNutri, $vrpefatcorse);

                    }
                    return true;
                }

            }
        }

    }

    public static function atualizaTodosProdutosServico($nrorg, $cdprato, $filial, $idLiqBrut, $servico)
    {

        /** @var $registro Receserv */
        $registro = Receserv::getObjectsByWhere('CDPRATO = ? AND NRORG = ?', array($cdprato, $nrorg));
        /** @var $linha Receserv */
        foreach ($registro as $linha) {
            $filAux           = $linha->getCdfilauxprat();
            $prato            = $linha->getCdprato();
            $produto          = $linha->getCdproduto();

            $validacao = self::atualizaFatorCorrecaoServico($nrorg, $filial, $prato, $filAux, $produto, $idLiqBrut, $servico);

        }
        if ($validacao == true) {
            $msgsucesso = tk_get_message_by_name('msg_importacaoSucesso');
            tk_message($msgsucesso);
        }

    }

    public static function salvaRegistro($produto, $cdprato, $cdfilial, $cdservfili, $cdfilauxprat, $nrorg, $nrorginclusao, $dtinclusao,
                                         $cdoperinclusao, $nrorgultatu, $cdoperultatu, $dtultatu, $qtdminima, $qtdmaxima, $idativo)
    {

        /** @var $registro Limpepra */
        $registro = new Limpepra();

        /** Converte os campos de data de string para objetos */
        $dtultatualizacao = DateUtil::getDataDeString($dtultatu, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
        $datainclusao = DateUtil::getDataDeString($dtinclusao, DateUtil::FORMATO_BRASILEIRO_DATAHORA);

        try {
            $conn = Registry::getEntityManager();
            $conn->beginTransaction();
            $registro->insereDados($produto, $cdprato, $cdfilial, $cdservfili, $cdfilauxprat, $nrorg, $nrorginclusao, $datainclusao,
                $cdoperinclusao, $nrorgultatu, $cdoperultatu, $dtultatualizacao, $qtdminima, $qtdmaxima, $idativo);
            $conn->flush();
            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            tk_message($e->getMessage(), 'error');
            return false;
        }

    }

    public static function excluiRegistroCompleto($nrorg, $cdfilial, $cdprato, $servico, $cdfilauxiliar)
    {

        /** @var $registro Limpepra */
        $retorno = true;
        $registro = Limpepra::getObjectsByWhere('CDFILIAL = ? AND CDSERVFILI = ? AND CDPRATO = ? AND CDFILAUXPRAT = ? AND NRORG = ?',
            array($cdfilial, $servico, $cdprato, $cdfilauxiliar, $nrorg));

        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        foreach ($registro as $linha) {
            try {
                $linha->delete();
                $conn->flush();
            } catch (\Exception $e) {
                $conn->rollback();
                tk_message($e->getMessage(), 'error');
                $retorno = false;
            }
        }
        $conn->commit();
        return $retorno;

    }

    public static function atualizaQtdePratoPadrao($tipoajuste, $valor, $pratos, $nrorg, $cdfilauxprat)
    {

        $listaPratos = '';
        $valorAjustado = '';

        foreach ($pratos as $prato) {
            $listaPratos .= $prato['CDPRATO'] . ",";
        }
//
        /** @var $objetos Pratpadr[] */
        $objetos = Pratpadr::getObjectsByWhere("NRORG = ?  AND INSTR ((?) , CDPRATO) <> 0",
                                array($nrorg, $listaPratos));


        try {
            $conn = Registry::getEntityManager();
            $conn->beginTransaction();

            if (is_array($objetos) === true && !empty($objetos)) {
                foreach ($objetos as $objeto) {
                    if($tipoajuste == self::PERCENTUAL) {

                        $quantidade = $objeto->getQtpadraoprat();
                        $valorAjustado = self::calculaPorcentagemAjuste($valor, $quantidade);
                    }
                    else if ($tipoajuste == self::QT_PESSOA){
                        $valorAjustado = $valor;
                    }
                    $objeto->atualizaQtdePratoPadrao($valorAjustado);
                    $conn->flush();
                }
            }
            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            tk_message($e->getMessage(), 'error');
            return false;
        }



    }

    public static function atualizaQtdePratoLocal ($tipoajuste, $valor, $pratos, $nrorg, $cdfilauxprat, $cdfilial) {
        $listaPratos = '';
        $valorAjustado = '';

        foreach ($pratos as $prato) {
            $listaPratos .= $prato['CDPRATO'] . ",";
        }
//
        /** @var $objetos Pratfili[] */
        $objetos = Pratfili::getObjectsByWhere("NRORG = ?  AND INSTR ((?) , CDPRATO) <> 0 AND CDFILIAL = ?",
                                                array($nrorg, $listaPratos, $cdfilial));


        try {
            $conn = Registry::getEntityManager();
            $conn->beginTransaction();

            if (is_array($objetos) === true && !empty($objetos)) {
                foreach ($objetos as $objeto) {
                    if($tipoajuste == self::PERCENTUAL) {
                        $quantidade = $objeto->getQtpadraofili();;
                        $valorAjustado = self::calculaPorcentagemAjuste($valor, $quantidade);
                    }
                    else if ($tipoajuste == self::QT_PESSOA){
                        $valorAjustado = $valor;
                    }
                    $objeto->atualizaQtdePratoLocal($valorAjustado);
                    $conn->flush();
                }
            }
            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            tk_message($e->getMessage(), 'error');
            return false;
        }
    }

    public static function atualizaQtdePratoServico ($tipoajuste, $valor, $pratos, $nrorg, $cdfilauxprat, $cdfilial, $cdservfili) {
        $listaPratos = '';
        $valorAjustado = '';

        foreach ($pratos as $prato) {
            $listaPratos .= $prato['CDPRATO'] . ",";
        }
//
        /** @var $objetos Pratserv[] */
        $objetos = Pratserv::getObjectsByWhere("NRORG = ?  AND INSTR ((?) , CDPRATO) <> 0 AND CDFILIAL = ?
                    AND CDSERVFILI = ?", array($nrorg, $listaPratos, $cdfilial, $cdservfili));


        try {
            $conn = Registry::getEntityManager();
            $conn->beginTransaction();

            if (is_array($objetos) === true && !empty($objetos)) {
                foreach ($objetos as $objeto) {
                    if($tipoajuste == self::PERCENTUAL) {
                        $quantidade = $objeto->getQtpadraoserv();
                        $valorAjustado = self::calculaPorcentagemAjuste($valor, $quantidade);
                    }
                    else if ($tipoajuste == self::QT_PESSOA){
                        $valorAjustado = $valor;
                    }
                    $objeto->atualizaQtdePratoServico($valorAjustado);
                    $conn->flush();
                }
            }
            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            tk_message($e->getMessage(), 'error');
            return false;
        }
    }


    public static function calculaPorcentagemAjuste($valor, $quantidade)
    {
        $valorReajustado = (($valor) * ($quantidade) / 100) + $quantidade;
        return $valorReajustado;


    }

    public static function inserePratoPadrao ($pratos) {

       foreach ($pratos as $prato) {
        $cdfilauxprat = $prato['CDFILAUXDEST'];
        $cdprato      = $prato['CDPRATODEST'];
        $nrcorpri     = $prato['NRCORPRI'];
        $nrcorseq     = $prato['NRCORSEQ'];
        $nrcoraux     = $prato['NRCORAUX'];
        $qtpadrao     = $prato['QTPADRAO'];
        $qtcontai     = $prato['QTCONTAI'];
        $hrvalcon     = $prato['HRVALCON'];
        $nmfantasia   = $prato['NMFANTASIA'];
        $qtcarbo      = $prato['QTCARBO'];
        $qtlipid      = $prato['QTLIPID'];
        $qtprote      = $prato['QTPROTE'];
        $qtcalor      = $prato['QTCALOR'];
        $qtalcoo      = $prato['QTALCOO'];

        //    Auditoria
        $dtinclusao     = DateUtil::getDataAtual();
        $cdoperinclusao = Ambiente::getCdOperador();
        $dtultatu       = DateUtil::getDataAtual();
        $nrorg          = Ambiente::getNrorg();


        $conn = Registry::getEntityManager();

        try {

            $conn->beginTransaction();
            $registro = new Pratpadr();

            $registro->inserePratoPadrao($cdfilauxprat, $cdprato, $nrcorpri, $nrcorseq, $nrcoraux, $qtpadrao,
                $qtcontai, $hrvalcon, $nmfantasia, $qtcarbo, $qtlipid, $qtprote, $qtcalor, $qtalcoo, $nrorg,
                $dtinclusao, $dtultatu, $cdoperinclusao);
            $conn->flush();
            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            tk_message($e->getMessage(), 'error');
            return false;
        }
       }
    }

    public static function inserePratoFilial ($pratos) {

      foreach ($pratos as $prato) {
        $cdfilauxprat = $prato['CDFILAUXDEST'];
        $cdprato      = $prato['CDPRATODEST'];
        $cdfilial     = $prato['CDFILIDEST'];
        $nrcorpri     = $prato['NRCORPRI'];
        $nrcorseq     = $prato['NRCORSEQ'];
        $nrcoraux     = $prato['NRCORAUX'];
        $qtpadrao     = $prato['QTPADRAO'];
        $qtcontai     = $prato['QTCONTAI'];
        $hrvalcon     = $prato['HRVALCON'];
        $nmfantasia   = $prato['NMFANTASIA'];
        $qtcarbo      = $prato['QTCARBO'];
        $qtlipid      = $prato['QTLIPID'];
        $qtprote      = $prato['QTPROTE'];
        $qtcalor      = $prato['QTCALOR'];
        $qtalcoo      = $prato['QTALCOO'];

        //    Auditoria
        $dtinclusao     = DateUtil::getDataAtual();
        $cdoperinclusao = Ambiente::getCdOperador();
        $dtultatu       = DateUtil::getDataAtual();
        $nrorg          = Ambiente::getNrorg();


        $conn = Registry::getEntityManager();

        try {

            $conn->beginTransaction();

            $registro = new Pratfili();

            $registro->inserePratoFilial($cdfilauxprat, $cdprato, $cdfilial, $nrcorpri, $nrcorseq, $nrcoraux, $qtpadrao,
                            $qtcontai, $hrvalcon, $nmfantasia, $qtcarbo, $qtlipid, $qtprote, $qtcalor, $qtalcoo,$nrorg,
                            $dtinclusao, $dtultatu, $cdoperinclusao);
            $conn->flush();
            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            tk_message($e->getMessage(), 'error');
            return false;
        }
      }
    }

    public static function inserePratoServico ($pratos) {

      foreach ($pratos as $prato) {
       $cdfilauxprat = $prato['CDFILAUXDEST'];
       $cdprato      = $prato['CDPRATODEST'];
       $cdfilial     = $prato['CDFILIDEST'];
       $cdservico    = $prato['CDSERVFILI'];
       $nrcorpri     = $prato['NRCORPRI'];
       $nrcorseq     = $prato['NRCORSEQ'];
       $nrcoraux     = $prato['NRCORAUX'];
       $qtpadrao     = $prato['QTPADRAO'];
       $qtcontai     = $prato['QTCONTAI'];
       $hrvalcon     = $prato['HRVALCON'];
       $nmfantasia   = $prato['NMFANTASIA'];
       $qtcarbo      = $prato['QTCARBO'];
       $qtlipid      = $prato['QTLIPID'];
       $qtprote      = $prato['QTPROTE'];
       $qtcalor      = $prato['QTCALOR'];
       $qtalcoo      = $prato['QTALCOO'];

        // Auditoria
       $dtinclusao     = DateUtil::getDataAtual();
       $cdoperinclusao = Ambiente::getCdOperador();
       $dtultatu       = DateUtil::getDataAtual();
       $nrorg          = Ambiente::getNrorg();


        $conn = Registry::getEntityManager();

        try {
            $conn->beginTransaction();
            $registro = new Pratserv();

            $registro->inserePratoServico($cdfilauxprat, $cdprato, $cdfilial, $cdservico, $nrcorpri, $nrcorseq, $nrcoraux, $qtpadrao,
                   $qtcontai, $hrvalcon, $nmfantasia, $qtcarbo, $qtlipid, $qtprote, $qtcalor, $qtalcoo, $nrorg,$dtinclusao, $dtultatu,
                   $cdoperinclusao);
            $conn->flush();
            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            tk_message($e->getMessage(), 'error');
            return false;
        }
      }
    }
	
	public static function concatenaFilialServico($nrorg, $servico, $filial){
        if ($filial == 'N') {
            tk_abort(tk_get_message_by_name('msg_ger04102_000_Filial_N'));
        } else {
            if ($filial == 'A') {
                if ($servico === 'A') {
                    /**
                     * Busca todos os serviços da tabela
                     * @var  $servicos \Geral\Entities\Servico[]
                     */
                    $servicos = \Geral\Entities\Servico::getObjectsByWhere(
                        'NRORG = ? AND INSTR ((?), CDFILIAL) <>  0', array($nrorg, $filial)
                    );
                    $listaConcatenada = '_';
                    foreach ($servicos as $servico) {
                        $listaConcatenada .= $servico->getCdfilial() . $servico->getCdservfili(). '_';
                    }
                    tk_set_context_vars('LISTAFILISERV', 'win_pla01308_000', $listaConcatenada);
                } else {
                    if ($servico === 'N') {
                        tk_abort(
                            tk_get_message_by_name('msg_ger04102_000_Servico_N')
                        );
                    } else {
                        $listaConcatenada = '_';
                        foreach ($servico as $servfili) {
                            $listaConcatenada .= $servfili['cdfilial'] . $servfili['cdservfili']. '_';
                        }
                        tk_set_context_vars('LISTAFILISERV', 'win_pla01308_000', $listaConcatenada);
                    }
                }
            } else {
                $listaConcatenada = '_';
                foreach ($filial as $fili) {
                    if ($servico === 'A') {
                        /**
                         * Busca a filial e os seus respectivos serviços
                         * @var  $servicos \Geral\Entities\Servico[]
                         */
                        $servicos = \Geral\Entities\Servico::getObjectsByWhere(
                            'CDFILIAL = ? AND NRORG = ? AND HRINICATEN IS NOT NULL',
                            array($fili, $nrorg)
                        );

                        foreach ($servicos as $serv) {
                            $listaConcatenada
                                .= $serv->getCdfilial() . $serv->getCdservfili(). '_';
                        }
                        tk_set_context_vars('LISTAFILISERV', 'win_pla01308_000', $listaConcatenada);
                    } else {
                        if ($servico === 'N') {
                            tk_abort(
                                tk_get_message_by_name(
                                    'msg_ger04102_000_Servico_N'
                                )
                            );
                        } else {
                            $listaConcatenada = '_';
                            foreach ($servico as $servfili) {
                                $listaConcatenada .= $servfili['cdfilial'] . $servfili['cdservfili']. '_';
                            }
                            tk_set_context_vars('LISTAFILISERV', 'win_pla01308_000', $listaConcatenada);
                        }
                    }
                }
            }
        }
    }

    public static function excluiProdutoReceitaPadrao($cdprato, $cdproduto, $nrorg)
    {

        /** @var $registro Recepadr */
        $retorno = true;
        $registro = Recepadr::getObjectByWhere('CDPRATO = ? AND CDPRODUTO = ? AND NRORG = ?', array($cdprato, $cdproduto, $nrorg));

        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        try {
            $registro->delete();
            $conn->flush();
        } catch (\Exception $e) {
            $conn->rollback();
            tk_message($e->getMessage(), 'error');
            $retorno = false;
        }

        $conn->commit();
        return $retorno;

    }

    public static function excluiProdutoReceitaLocal($cdprato, $cdproduto, $filial, $nrorg)

    {

        /** @var $registro Recefili */
        $retorno = true;
        $registro = Recefili::getObjectByWhere('CDPRATO = ? AND CDPRODUTO = ? AND NRORG = ? AND CDFILIAL = ?', array($cdprato, $cdproduto, $nrorg, $filial));

        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        try {
            $registro->delete();
            $conn->flush();
        } catch (\Exception $e) {
            $conn->rollback();
            tk_message($e->getMessage(), 'error');
            $retorno = false;
        }

        $conn->commit();
        return $retorno;

    }

    public static function excluiProdutoReceitaServico($cdprato, $cdproduto, $filial, $servico, $nrorg)
    {

        /** @var $registro Receserv */
        $retorno = true;
        $registro = Receserv::getObjectByWhere('CDPRATO = ? AND CDPRODUTO = ? AND NRORG = ? AND CDFILIAL = ? AND CDSERVFILI = ?',
            array($cdprato, $cdproduto, $nrorg, $filial, $servico));

        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        try {
            $registro->delete();
            $conn->flush();
        } catch (\Exception $e) {
            $conn->rollback();
            tk_message($e->getMessage(), 'error');
            $retorno = false;
        }

        $conn->commit();
        return $retorno;

    }
}